<?php

namespace MoDynamics365ObjectSync\Wrappers;

use wpdb;

class dbWrapper{

    private static $instance;

    public static function getWrapper(){
        if(!isset(self::$instance)){
            $class = __CLASS__;
            self::$instance = new $class;
        }
        return self::$instance;
    }

    public static function mo_dcrm_create_cf7_object_map_table(){
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();
        $table_name = $wpdb->prefix . dbConstants::CF7_OBJECT_MAPPING_TABLE;

        $sql = "CREATE TABLE $table_name (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            cf7_form_id text NOT NULL,
            form_fields text NOT NULL,
            dynamics_object_id text NOT NULL,
            object_attrs longtext,
            field_map longtext, 
            PRIMARY KEY id (id),
            KEY cf7_form_id (cf7_form_id)
        ) $charset_collate;";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
    }

    public static function mo_dcrm_update_cf7_object_map_table_row($key,$value,$id = 0){
        global $wpdb;
        $table = $wpdb->prefix.dbConstants::CF7_OBJECT_MAPPING_TABLE;

        $data = array($key => $value);
        $format = array('%s');

        $has_old_value = dbWrapper::mo_dcrm_get_cf7_object_map_table_row_id($id);

        if($has_old_value){
            $wpdb->update( $table, $data, array('id'=>$has_old_value), $format );
        }else{
            $wpdb->insert($table,$data,$format);
        }
        
    }

    public static function mo_dcrm_table_exists($table){
        global $wpdb;
        $result=$wpdb->get_results($wpdb->prepare("SHOW TABLES LIKE %s",$table),ARRAY_A);
        return !empty($result);
    }

    public static function mo_dcrm_get_new_id(){
        global $wpdb;
        $table= $wpdb->prefix.dbConstants::CF7_OBJECT_MAPPING_TABLE;

        if(!dbWrapper::mo_dcrm_table_exists($table))
            return 0;

        $db=$wpdb->dbname;
        $result=$wpdb->get_results($wpdb->prepare("SELECT `AUTO_INCREMENT` FROM  INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s",$db,$table),ARRAY_A);
        $next_id=$result[0]['AUTO_INCREMENT'];
        return $next_id;
    }

    public static function mo_dcrm_delete_row($id){
        global $wpdb;
        $wpdb->delete($wpdb->prefix.dbConstants::CF7_OBJECT_MAPPING_TABLE, array( 'id' => $id ));
        return;
    }

    public static function mo_dcrm_get_cf7_object_map_table_row_id($id){
        global $wpdb;
        $table = $wpdb->prefix.dbConstants::CF7_OBJECT_MAPPING_TABLE;
        $result = $wpdb->get_results( $wpdb->prepare("SELECT id FROM %5s where id = %s",$table,$id));
        if(empty($result))
            return false;

        return $result[0]->id;

    }

    public static function mo_dcrm_get_cf7_object_map_table_row($id){
        global $wpdb;
        $table = $wpdb->prefix.dbConstants::CF7_OBJECT_MAPPING_TABLE;
        $result = $wpdb->get_results($wpdb->prepare("SELECT * FROM %5s where id = %s",$table,$id), ARRAY_A );
        
        if(empty($result))
            return [];

        return $result[0];

    }

    public static function mo_dcrm_get_cf7_object_map_table_rows_by_key($key,$id,$select = '*'){
        global $wpdb;
        $table = $wpdb->prefix.dbConstants::CF7_OBJECT_MAPPING_TABLE;
        $result = $wpdb->get_results($wpdb->prepare("SELECT  %5s FROM %5s where %5s = %s",$select, $table,$key, $id), ARRAY_A );
        
        if(empty($result))
            return [];

        return $result;
    }

     public static function mo_dcrm_get_all_cf7_object_entity_config(){
        global $wpdb;
        $table=$wpdb->prefix.dbConstants::CF7_OBJECT_MAPPING_TABLE;

        if(!dbWrapper::mo_dcrm_table_exists($table))
            return [];

        $result=$wpdb->get_results($wpdb->prepare("SELECT id, cf7_form_id, dynamics_object_id, field_map from %5s",$table),ARRAY_A);
        return $result;
     }
}